@@ -3,7 +3,7 @@
  *                                                                 *
  * Copyright (C) 2003 Paul Pogonyshev.                             *
  * Copyright (C) 2004 Paul Pogonyshev and Martin Holters.          *
- * Copyright (C) 2005 Paul Pogonyshev.                             *
+ * Copyright (C) 2005, 2006 Paul Pogonyshev.                       *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -27,6 +27,7 @@
 #include "utils.h"
 
 #include <assert.h>
+#include <math.h>
 
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
@@ -311,8 +312,8 @@ board_play_move (Board *board, int color, ...)
   if (board->move_stack_pointer == board->move_stack_end)
     board_increase_move_stack_size (board);
 
-  memcpy (board->move_stack_pointer, board->grid,
-	  BOARD_GRID_SIZE * sizeof (char));
+  memcpy (((BoardStackEntry *) board->move_stack_pointer).grid_copy,
+	  board->grid, BOARD_GRID_SIZE * sizeof (char));
 #endif
 
   va_start (move, color);
@@ -343,8 +344,8 @@ board_apply_changes
   if (board->move_stack_pointer == board->move_stack_end)
     board_increase_move_stack_size (board);
 
-  memcpy (board->move_stack_pointer, board->grid,
-	  BOARD_GRID_SIZE * sizeof (char));
+  memcpy (((BoardStackEntry *) board->move_stack_pointer).grid_copy,
+	  board->grid, BOARD_GRID_SIZE * sizeof (char));
 #endif
 
   num_changes = 0;
@@ -396,8 +397,8 @@ board_add_dummy_move_entry (Board *board)
   if (board->move_stack_pointer == board->move_stack_end)
     board_increase_move_stack_size (board);
 
-  memcpy (board->move_stack_pointer, board->grid,
-	  BOARD_GRID_SIZE * sizeof (char));
+  memcpy (((BoardStackEntry *) board->move_stack_pointer).grid_copy,
+	  board->grid, BOARD_GRID_SIZE * sizeof (char));
 #endif
 
   game_info[board->game].add_dummy_move_entry (board);
@@ -428,7 +429,8 @@ board_undo (Board *board, int num_undos)
 
 #if BOARD_VALIDATION_LEVEL == 2
   {
-    const char * grid_copy = (const char *) board->move_stack_pointer;
+    const char * grid_copy
+      = ((BoardStackEntry *) board->move_stack_pointer).grid_copy;
     int x;
     int y;
 
@@ -443,6 +445,28 @@ board_undo (Board *board, int num_undos)
 }
 
 
+int
+board_get_move_number (const Board *board, int num_moves_backward)
+{
+  assert (board);
+  assert (num_moves_backward >= 0);
+  assert (((char *) board->move_stack_pointer
+	   - (num_moves_backward * game_info[board->game].stack_entry_size))
+	  >= (char *) board->move_stack);
+
+  if (num_moves_backward == 0)
+    return board->move_number;
+  else {
+    const BoardStackEntry *stack_entry
+      = ((const BoardStackEntry *)
+	 ((const char *) board->move_stack_pointer
+	  - (num_moves_backward * game_info[board->game].stack_entry_size)));
+
+    return stack_entry->move_number;
+  }
+}
+
+
 inline void
 board_undo_changes (Board *board, int num_changes)
 {
@@ -792,9 +816,9 @@ game_from_game_name (const char *game_name, int case_sensitive)
 }
 
 
-int
+void
 game_format_point (Game game, int board_width, int board_height,
-		   char *buffer, int x, int y)
+		   StringBuffer *buffer, int x, int y)
 {
   assert (game >= FIRST_GAME && GAME_IS_SUPPORTED (game));
   assert (BOARD_MIN_WIDTH <= board_width && board_width <= BOARD_MAX_WIDTH);
@@ -803,19 +827,18 @@ game_format_point (Game game, int board_width, int board_height,
   assert (buffer);
   assert (ON_SIZED_GRID (board_width, board_height, x, y));
 
-  *buffer = game_info[game].horizontal_coordinates[x];
-  return (1 + utils_ncprintf (buffer + 1, 3, "%d",
-			      (game_info[game].reversed_vertical_coordinates
-			       ? board_height - y : y + 1)));
+  string_buffer_cprintf (buffer, "%c%d", 
+			 game_info[game].horizontal_coordinates[x],
+			 (game_info[game].reversed_vertical_coordinates
+			  ? board_height - y : y + 1));
 }
 
 
-int
+void
 game_format_position_list (Game game, int board_width, int board_height,
-			   char *buffer,
+			   StringBuffer *buffer,
 			   const BoardPositionList *position_list)
 {
-  char *buffer_pointer = buffer;
   int k;
 
   assert (position_list);
@@ -823,49 +846,85 @@ game_format_position_list (Game game, int board_width, int board_height,
   for (k = 0; k < position_list->num_positions; k++) {
     int pos = position_list->positions[k];
 
-    buffer_pointer += game_format_point (game, board_width, board_height,
-					 buffer_pointer,
-					 POSITION_X (pos), POSITION_Y (pos));
-    *buffer_pointer++ = ' ';
-  }
+    if (k > 0)
+      string_buffer_add_character (buffer, ' ');
 
-  *--buffer_pointer = 0;
-  return buffer_pointer - buffer;
+    game_format_point (game, board_width, board_height, buffer,
+		       POSITION_X (pos), POSITION_Y (pos));
+  }
 }
 
 
-int
+void
 game_format_move (Game game, int board_width, int board_height,
-		  char *buffer, ...)
+		  StringBuffer *buffer, ...)
 {
   va_list move;
-  int num_characters;
 
   assert (game >= FIRST_GAME && GAME_IS_SUPPORTED (game));
 
   va_start (move, buffer);
-  num_characters = game_info[game].format_move (board_width, board_height,
-						buffer, move);
+  game_info[game].format_move (board_width, board_height, buffer, move);
   va_end (move);
-
-  buffer[num_characters] = 0;
-  return num_characters;
 }
 
 
-int
+void
 game_format_move_valist (Game game, int board_width, int board_height,
-			 char *buffer, va_list move)
+			 StringBuffer *buffer, va_list move)
 {
-  int num_characters;
-
   assert (game >= FIRST_GAME && GAME_IS_SUPPORTED (game));
 
-  num_characters = game_info[game].format_move (board_width, board_height,
-						buffer, move);
-  buffer[num_characters] = 0;
+  game_info[game].format_move (board_width, board_height, buffer, move);
+}
 
-  return num_characters;
+
+void
+game_format_score_difference (Game game, StringBuffer *buffer, double score)
+{
+  /* We may want to distinguish based on game later. */
+  UNUSED (game);
+  assert (game >= FIRST_GAME && GAME_IS_SUPPORTED (game));
+
+  if (fabs (score - floor (score + 0.005)) >= 0.005) {
+    int num_digits
+      = ((score + 0.005) * 10 - floor ((score + 0.005) * 10) >= 0.1 ? 2 : 1);
+
+    if (score > 0.0) {
+      /* TRANSLATORS: This is only used for non-integral scores.  You
+	 should also add translation of `points' if it is required by
+	 your language. */
+      string_buffer_printf (buffer, _("Black wins by %.*f"),
+			    num_digits, score);
+    }
+    else {
+      /* TRANSLATORS: This is only used for non-integral scores.  You
+	 should also add translation of `points' if it is required by
+	 your language. */
+      string_buffer_printf (buffer, _("White wins by %.*f"),
+			    num_digits, -score);
+    }
+  }
+  else {
+    int integral_score = (int) floor (score + 0.005);
+
+    if (integral_score > 0) {
+      /* TRANSLATORS: You should add translation of `points' if it is
+	 required by your language. */
+      string_buffer_printf (buffer,
+			    ngettext ("Black wins by %d", "Black wins by %d",
+				      integral_score),
+			    integral_score);
+    }
+    else {
+      /* TRANSLATORS: You should add translation of `points' if it is
+	 required by your language. */
+      string_buffer_printf (buffer,
+			    ngettext ("White wins by %d", "White wins by %d",
+				      -integral_score),
+			    -integral_score);
+    }
+  }
 }
 
 
