@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
  * This file is part of Quarry.                                    *
  *                                                                 *
- * Copyright (C) 2003, 2004, 2005 Paul Pogonyshev.                 *
+ * Copyright (C) 2003, 2004, 2005, 2006 Paul Pogonyshev.           *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -22,7 +22,6 @@
 
 #include "gtk-new-game-dialog.h"
 
-#include "gtk-assistant.h"
 #include "gtk-configuration.h"
 #include "gtk-control-center.h"
 #include "gtk-games.h"
@@ -31,6 +30,7 @@
 #include "gtk-named-vbox.h"
 #include "gtk-preferences.h"
 #include "gtk-utils.h"
+#include "quarry-assistant.h"
 #include "quarry-stock.h"
 #include "time-control.h"
 #include "sgf.h"
@@ -38,7 +38,6 @@
 #include "game-info.h"
 
 #include <gtk/gtk.h>
-#include <assert.h>
 
 
 static void	      gtk_new_game_dialog_init (GtkNewGameDialog *dialog);
@@ -83,7 +82,7 @@ gtk_new_game_dialog_get_type (void)
       NULL
     };
 
-    new_game_dialog_type = g_type_register_static (GTK_TYPE_ASSISTANT,
+    new_game_dialog_type = g_type_register_static (QUARRY_TYPE_ASSISTANT,
 						   "GtkNewGameDialog",
 						   &new_game_dialog_info, 0);
   }
@@ -125,8 +124,8 @@ gtk_new_game_dialog_init (GtkNewGameDialog *dialog)
   gtk_window_set_title (GTK_WINDOW (dialog), _("New Game"));
   gtk_utils_make_window_only_horizontally_resizable (GTK_WINDOW (dialog));
 
-  gtk_assistant_set_user_data (&dialog->assistant, dialog);
-  gtk_assistant_set_finish_button (&dialog->assistant, QUARRY_STOCK_PLAY);
+  quarry_assistant_set_user_data (&dialog->assistant, dialog);
+  quarry_assistant_set_finish_button (&dialog->assistant, QUARRY_STOCK_PLAY);
 
   /* "Game & Players" page. */
 
@@ -284,11 +283,12 @@ gtk_new_game_dialog_init (GtkNewGameDialog *dialog)
     = gtk_utils_pack_in_box (GTK_TYPE_VBOX, QUARRY_SPACING_BIG,
 			     named_vbox, GTK_UTILS_FILL,
 			     hbox, GTK_UTILS_FILL, NULL);
-  gtk_assistant_add_page (&dialog->assistant, game_and_players_page,
-			  GTK_STOCK_REFRESH, _("Game & Players"), NULL, NULL);
-  gtk_assistant_set_page_help_link_id (&dialog->assistant,
-				       game_and_players_page,
-				       "new-game-dialog-game-and-players");
+  quarry_assistant_add_page (&dialog->assistant, game_and_players_page,
+			     GTK_STOCK_REFRESH, _("Game & Players"),
+			     NULL, NULL);
+  quarry_assistant_set_page_help_link_id (&dialog->assistant,
+					  game_and_players_page,
+					  "new-game-dialog-game-and-players");
   gtk_widget_show_all (game_and_players_page);
 
   if (game_index != GTK_GAME_UNSUPPORTED) {
@@ -345,20 +345,12 @@ gtk_new_game_dialog_init (GtkNewGameDialog *dialog)
 					    QUARRY_SPACING_SMALL);
     rules_vbox = GTK_BOX (rules_vbox_widget);
 
-    /* Board size spin button and a label for it. */
-    dialog->board_sizes[k] =
-      gtk_games_create_board_size_adjustment (k, initial_board_size);
-    board_size_spin_button
-      = gtk_utils_create_spin_button (dialog->board_sizes[k], 0.0, 0, TRUE);
-
-    label = gtk_utils_create_mnemonic_label (_("Board _size:"),
-					     board_size_spin_button);
-
-    /* Pack them into a box and add to "Game Rules" box. */
-    hbox = gtk_utils_pack_in_box (GTK_TYPE_HBOX, QUARRY_SPACING,
-				  label, 0,
-				  board_size_spin_button, GTK_UTILS_FILL,
-				  NULL);
+    /* Board size controls. */
+    dialog->board_sizes[k]
+      = gtk_games_create_board_size_adjustment (k, initial_board_size);
+    hbox
+      = gtk_games_create_board_size_selector_box (k, dialog->board_sizes[k],
+						  &board_size_spin_button);
     gtk_box_pack_start (rules_vbox, hbox, FALSE, TRUE, 0);
 
     /* A combo box (option menu for pre-2.4) for selecting time
@@ -607,15 +599,15 @@ gtk_new_game_dialog_init (GtkNewGameDialog *dialog)
   }
 
   /* Finally, add the game rules notebook as an assistant page. */
-  gtk_assistant_add_page (&dialog->assistant, games_notebook,
-			  GTK_STOCK_PREFERENCES, _("Game Rules"),
-			  ((GtkAssistantPageShownCallback)
-			   show_game_specific_rules),
-			  ((GtkAssistantPageAcceptableCallback)
-			   instantiate_players));
-  gtk_assistant_set_page_help_link_id_callback
+  quarry_assistant_add_page (&dialog->assistant, games_notebook,
+			     GTK_STOCK_PREFERENCES, _("Game Rules"),
+			     ((QuarryAssistantPageShownCallback)
+			      show_game_specific_rules),
+			     ((QuarryAssistantPageAcceptableCallback)
+			      instantiate_players));
+  quarry_assistant_set_page_help_link_id_callback
     (&dialog->assistant, games_notebook,
-     (GtkAssistantPageHelpLinkIDCallback) get_game_rules_help_link_id);
+     (QuarryAssistantPageHelpLinkIDCallback) get_game_rules_help_link_id);
   gtk_widget_show_all (games_notebook);
 }
 
@@ -777,7 +769,9 @@ get_game_rules_help_link_id (GtkNewGameDialog *dialog)
     return "new-game-dialog-reversi-rules";
 
   default:
-    assert (0);
+    g_warning ("unhandled game %s",
+	       INDEX_TO_GAME_NAME (get_selected_game (dialog)));
+    return NULL;
   }
 }
 
@@ -822,6 +816,8 @@ begin_game (GtkEnginesInstantiationStatus status, gpointer user_data)
   TimeControl *time_control;
   SgfGameTree *game_tree;
   SgfCollection *sgf_collection;
+  Board *board;
+  SgfBoardState board_state;
   GtkWidget *goban_window;
   int k;
 
@@ -840,6 +836,12 @@ begin_game (GtkEnginesInstantiationStatus status, gpointer user_data)
   game_tree = sgf_game_tree_new_with_root (game, board_size, board_size, 1);
   game_tree->char_set = utils_duplicate_string ("UTF-8");
 
+  sgf_collection = sgf_collection_new ();
+  sgf_collection_add_game_tree (sgf_collection, game_tree);
+
+  board = board_new (game, board_size, board_size);
+  sgf_utils_enter_tree (game_tree, board, &board_state);
+
   sgf_node_add_text_property (game_tree->root, game_tree,
 			      SGF_RESULT, utils_duplicate_string ("Void"), 0);
 
@@ -920,6 +922,7 @@ begin_game (GtkEnginesInstantiationStatus status, gpointer user_data)
   }
   else {
     switch (time_control_configuration->type) {
+      /* FIXME: Should be an enumeration, not hardwired constants. */
     case 1:
       time_control_configuration->game_time_limit
 	= gtk_adjustment_get_value (time_control_data->game_time_limit);
@@ -953,17 +956,14 @@ begin_game (GtkEnginesInstantiationStatus status, gpointer user_data)
       break;
 
     default:
-      assert (0);
+      g_assert_not_reached ();
     }
 
     time_control_save_settings_in_sgf_node (time_control,
 					    game_tree->current_node,
-					    game_tree);
+					    game_tree, 0);
   }
 
-  sgf_collection = sgf_collection_new ();
-  sgf_collection_add_game_tree (sgf_collection, game_tree);
-
   configuration_set_string_value (&new_game_configuration.game_name,
 				  game_info[game].name);
 
@@ -976,6 +976,10 @@ begin_game (GtkEnginesInstantiationStatus status, gpointer user_data)
 				    engine_screen_names[k]);
   }
 
+  board_delete (board);
+  game_tree->board	 = NULL;
+  game_tree->board_state = NULL;
+
   goban_window = gtk_goban_window_new (sgf_collection, NULL);
   gtk_goban_window_enter_game_mode (GTK_GOBAN_WINDOW (goban_window),
 				    dialog->players[BLACK_INDEX],
