@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
  * This file is part of Quarry.                                    *
  *                                                                 *
- * Copyright (C) 2005 Paul Pogonyshev.                             *
+ * Copyright (C) 2005, 2006 Paul Pogonyshev.                       *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -27,6 +27,7 @@
 #include "gtk-freezable-spin-button.h"
 #include "gtk-games.h"
 #include "gtk-goban-window.h"
+#include "gtk-help.h"
 #include "gtk-named-vbox.h"
 #include "gtk-utils.h"
 #include "quarry-stock.h"
@@ -35,7 +36,6 @@
 #include "game-info.h"
 
 #include <gtk/gtk.h>
-#include <assert.h>
 
 
 static void	gtk_new_game_record_dialog_class_init
@@ -152,15 +152,10 @@ gtk_new_game_record_dialog_init (GtkNewGameRecordDialog *dialog)
 
     dialog->board_sizes[k]
       = gtk_games_create_board_size_adjustment (k, default_board_size);
+    hbox = gtk_games_create_board_size_selector_box (k, dialog->board_sizes[k],
+						     &spin_button);
 
-    spin_button = gtk_utils_create_spin_button (dialog->board_sizes[k],
-						0.0, 0, TRUE);
     gtk_size_group_add_widget (spin_button_size_group, spin_button);
-
-    label = gtk_utils_create_mnemonic_label (_("Board _size:"), spin_button);
-
-    hbox = gtk_utils_pack_in_box (GTK_TYPE_HBOX, QUARRY_SPACING,
-				  label, 0, spin_button, GTK_UTILS_FILL, NULL);
     gtk_size_group_add_widget (height_size_group, hbox);
 
     rules_named_vbox = gtk_named_vbox_new (_(game_rules_labels[k]), FALSE,
@@ -323,7 +318,8 @@ gtk_new_game_record_dialog_init (GtkNewGameRecordDialog *dialog)
 
   gtk_dialog_add_buttons (&dialog->dialog,
 			  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			  QUARRY_STOCK_CREATE, GTK_RESPONSE_OK, NULL);
+			  QUARRY_STOCK_CREATE, GTK_RESPONSE_OK,
+			  GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
   gtk_dialog_set_default_response (&dialog->dialog, GTK_RESPONSE_OK);
 }
 
@@ -450,6 +446,10 @@ gtk_new_game_record_dialog_response (GtkDialog *dialog, gint response_id)
     gtk_goban_window_enter_game_record_mode (GTK_GOBAN_WINDOW (goban_window));
     gtk_window_present (GTK_WINDOW (goban_window));
   }
+  else if (response_id == GTK_RESPONSE_HELP) {
+    gtk_help_display ("new-game-record-dialog");
+    return;
+  }
 
   gtk_widget_destroy (GTK_WIDGET (dialog));
 }
