@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
  * This file is part of Quarry.                                    *
  *                                                                 *
- * Copyright (C) 2003, 2004, 2005 Paul Pogonyshev.                 *
+ * Copyright (C) 2003, 2004, 2005, 2006 Paul Pogonyshev.           *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -27,7 +27,6 @@
 #include "utils.h"
 
 #include <assert.h>
-#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -471,17 +470,19 @@ sgf_node_new (SgfGameTree *tree, SgfNode *parent)
   SgfNode *node;
 
   node = memory_pool_alloc (&tree->node_pool);
-  node->parent		  = parent;
-  node->child		  = NULL;
-  node->next		  = NULL;
-  node->current_variation = NULL;
+
+  node->parent			  = parent;
+  node->child			  = NULL;
+  node->next			  = NULL;
+  node->current_variation	  = NULL;
 
   node->is_collapsed		  = 0;
   node->has_intermediate_map_data = 0;
 
-  node->move_color = EMPTY;
+  node->to_play_color		  = EMPTY;
+  node->move_color		  = EMPTY;
 
-  node->properties = NULL;
+  node->properties		  = NULL;
 
   return node;
 }
@@ -581,7 +582,9 @@ sgf_node_duplicate (const SgfNode *node, SgfGameTree *tree, SgfNode *parent)
   assert (node);
   assert (tree);
 
-  node_copy->move_color = node->move_color;
+  node_copy->to_play_color = node->to_play_color;
+  node_copy->move_color	   = node->move_color;
+
   if (IS_STONE (node->move_color)) {
     node_copy->move_point = node->move_point;
 
@@ -629,7 +632,7 @@ sgf_node_duplicate_recursively (const SgfNode *node, SgfGameTree *tree,
     /* Recurse for each branch. */
     for (link = &parent->child; node;
 	 node = node->next, link = & (*link)->next)
-      *link = sgf_node_duplicate (node, tree, parent);
+      *link = sgf_node_duplicate_recursively (node, tree, parent);
   }
 
   return node_copy;
@@ -1103,65 +1106,6 @@ sgf_node_add_pointer_property (SgfNode *node, SgfGameTree *tree,
 }
 
 
-int
-sgf_node_add_score_result (SgfNode *node, SgfGameTree *tree,
-			   double score, int overwrite)
-{
-  char *result;
-
-  if (score < -0.000005 || 0.000005 < score) {
-    char color_who_won = (score > 0.0 ? 'B' : 'W');
-
-    if (tree->game == GAME_GO
-	&& fabs (score - floor (score + 0.000005)) >= 0.000005)
-      result = utils_cprintf ("%c+%.f", color_who_won, fabs (score));
-    else
-      result = utils_cprintf ("%c+%d", color_who_won, (int) fabs (score));
-  }
-  else
-    result = utils_duplicate_string ("Draw");
-
-  return sgf_node_add_text_property (node, tree, SGF_RESULT, result,
-				     overwrite);
-}
-
-
-/* Add a text property to a given node.  If such a property already
- * exists, don't overwrite its value, but instead append new value to
- * the old one, putting `separator' in between.  If this property
- * doesn't yet exist, the `separator' is not used.  If `separator' is
- * NULL, then "\n\n" is used.
- *
- * Return non-zero if the value is appended (zero if added).
- */
-int
-sgf_node_append_text_property (SgfNode *node, SgfGameTree *tree,
-			       SgfType type, char *text, const char *separator)
-{
-  SgfProperty **link;
-
-  assert (node);
-  assert (property_info[type].value_type == SGF_SIMPLE_TEXT
-	  || property_info[type].value_type == SGF_FAKE_SIMPLE_TEXT
-	  || property_info[type].value_type == SGF_TEXT);
-
-  if (!sgf_node_find_property (node, type, &link)) {
-    *link = sgf_property_new (tree, type, *link);
-    (*link)->value.memory_block = text;
-
-    return 0;
-  }
-
-  (*link)->value.memory_block = utils_cat_strings ((*link)->value.memory_block,
-						   (separator
-						    ? separator : "\n\n"),
-						   text, NULL);
-  utils_free (text);
-
-  return 1;
-}
-
-
 
 /* Find and delete property of given type in the node.  Return nonzero
  * if succeded, or zero if there is no such property.
