@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
  * This file is part of Quarry.                                    *
  *                                                                 *
- * Copyright (C) 2003, 2004, 2005 Paul Pogonyshev.                 *
+ * Copyright (C) 2003, 2004, 2005, 2006 Paul Pogonyshev.           *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -24,9 +24,11 @@
 #include "sgf-privates.h"
 #include "sgf-undo.h"
 #include "board.h"
+#include "game-info.h"
 #include "utils.h"
 
 #include <assert.h>
+#include <math.h>
 
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
@@ -37,17 +39,26 @@ typedef int (* ValuesComparator) (const void *first_value,
 				  const void *second_value);
 
 
+static void	add_horizontal_coordinates (const SgfGameTree *tree,
+					    StringBuffer *buffer);
+static void	add_horizontal_line (const SgfGameTree *tree,
+				     StringBuffer *buffer, int sensei_style);
+
 static void	do_enter_tree (SgfGameTree *tree, SgfNode *down_to);
 
+static void	find_board_state_data (const SgfGameTree *tree,
+				       SgfNode *node,
+				       int need_color_to_move,
+				       int need_move_coordinates);
 static void	find_time_control_data (const SgfGameTree *tree,
 					SgfNode *upper_limit,
 					SgfNode *lower_limit);
-static void	determine_final_color_to_play (SgfGameTree *tree);
+static void	determine_final_color_to_play (const SgfGameTree *tree);
 
 static int	do_set_pointer_property (SgfNode *node, SgfGameTree *tree,
 					 SgfType type,
 					 ValuesComparator values_are_equal,
-					 void *new_value);
+					 void *new_value, int side_effect);
 static int	strings_are_equal (const void *first_string,
 				   const void *second_string);
 
@@ -69,54 +80,40 @@ sgf_utils_play_node_move (const SgfNode *node, Board *board)
 }
 
 
-int
+void
 sgf_utils_format_node_move (const SgfGameTree *tree, const SgfNode *node,
-			    char *buffer,
+			    StringBuffer *buffer,
 			    const char *black_string, const char *white_string,
 			    const char *pass_string)
 {
-  char *pointer = buffer;
-
   assert (tree);
   assert (node);
   assert (buffer);
   assert (IS_STONE (node->move_color));
 
   if (node->move_color == BLACK) {
-    if (black_string) {
-      pointer += strlen (black_string);
-      memcpy (buffer, black_string, pointer - buffer);
-    }
+    if (black_string)
+      string_buffer_cat_string (buffer, black_string);
   }
   else {
-    if (white_string) {
-      pointer += strlen (white_string);
-      memcpy (buffer, white_string, pointer - buffer);
-    }
+    if (white_string)
+      string_buffer_cat_string (buffer, white_string);
   }
 
   if (tree->game != GAME_AMAZONS) {
     if (tree->game != GAME_GO
-	|| !IS_PASS (node->move_point.x, node->move_point.y) || !pass_string) {
-      pointer += game_format_move (tree->game,
-				   tree->board_width, tree->board_height,
-				   pointer,
-				   node->move_point.x, node->move_point.y);
-    }
-    else {
-      strcpy (pointer, pass_string);
-      pointer += strlen (pass_string);
+	|| !IS_PASS (node->move_point.x, node->move_point.y)) {
+      game_format_move (tree->game, tree->board_width, tree->board_height,
+			buffer, node->move_point.x, node->move_point.y);
     }
+    else if (pass_string)
+      string_buffer_cat_string (buffer, pass_string);
   }
   else {
-    pointer += game_format_move (tree->game,
-				 tree->board_width, tree->board_height,
-				 pointer,
-				 node->move_point.x, node->move_point.y,
-				 node->data.amazons);
+    game_format_move (tree->game, tree->board_width, tree->board_height,
+		      buffer, node->move_point.x, node->move_point.y,
+		      node->data.amazons);
   }
-
-  return pointer - buffer;
 }
 
 
@@ -372,7 +369,7 @@ sgf_utils_switch_to_given_node (SgfGameTree *tree, SgfNode *node)
  * tree's game.  If `color' is `EMPTY', then the new variation will
  * not contain a move.
  */
-void
+SgfNode *
 sgf_utils_append_variation (SgfGameTree *tree, int color, ...)
 {
   SgfNode *new_node;
@@ -404,12 +401,14 @@ sgf_utils_append_variation (SgfGameTree *tree, int color, ...)
   sgf_utils_apply_undo_history_entry
     (tree, sgf_new_node_undo_history_entry_new (new_node));
   sgf_utils_end_action (tree);
+
+  return new_node;
 }
 
 
 int
 sgf_utils_apply_setup_changes (SgfGameTree *tree,
-			       const char grid[BOARD_GRID_SIZE])
+			       const char grid[BOARD_GRID_SIZE], int side_effect)
 {
   BoardPositionList *difference_lists[NUM_ON_GRID_VALUES];
   SgfNode *node;
@@ -460,20 +459,25 @@ sgf_utils_apply_setup_changes (SgfGameTree *tree,
   }
   else if (node->move_color != new_move_color) {
     SgfUndoHistoryEntry *entry
-      = sgf_change_node_move_color_undo_history_entry_new (node,
-							   new_move_color);
+      = (sgf_change_node_inlined_color_undo_history_entry_new
+	 (node, SGF_OPERATION_CHANGE_NODE_MOVE_COLOR, new_move_color,
+	  side_effect));
 
     sgf_utils_apply_undo_history_entry (tree, entry);
   }
 
   anything_changed |= (sgf_utils_set_list_of_point_property
-		       (node, tree, SGF_ADD_EMPTY, difference_lists[EMPTY]));
+		       (node, tree, SGF_ADD_EMPTY, difference_lists[EMPTY],
+			side_effect));
   anything_changed |= (sgf_utils_set_list_of_point_property
-		       (node, tree, SGF_ADD_BLACK, difference_lists[BLACK]));
+		       (node, tree, SGF_ADD_BLACK, difference_lists[BLACK],
+			side_effect));
   anything_changed |= (sgf_utils_set_list_of_point_property
-		       (node, tree, SGF_ADD_WHITE, difference_lists[WHITE]));
+		       (node, tree, SGF_ADD_WHITE, difference_lists[WHITE],
+			side_effect));
   anything_changed |= (sgf_utils_set_list_of_point_property
-		       (node, tree, SGF_ADD_ARROWS, difference_lists[ARROW]));
+		       (node, tree, SGF_ADD_ARROWS, difference_lists[ARROW],
+			side_effect));
 
   if (created_new_node) {
     tree->undo_history = undo_history;
@@ -495,7 +499,8 @@ sgf_utils_apply_setup_changes (SgfGameTree *tree,
 
 int
 sgf_utils_apply_markup_changes (SgfGameTree *tree,
-				const char markup_grid[BOARD_GRID_SIZE])
+				const char markup_grid[BOARD_GRID_SIZE],
+				int side_effect)
 {
   int num_positions[NUM_SGF_MARKUPS];
   int positions[NUM_SGF_MARKUPS][BOARD_MAX_POSITIONS];
@@ -538,19 +543,19 @@ sgf_utils_apply_markup_changes (SgfGameTree *tree,
 
   anything_changed |= (sgf_utils_set_list_of_point_property
 		       (tree->current_node, tree, SGF_MARK,
-			new_markup_lists[SGF_MARKUP_CROSS]));
+			new_markup_lists[SGF_MARKUP_CROSS], side_effect));
   anything_changed |= (sgf_utils_set_list_of_point_property
 		       (tree->current_node, tree, SGF_CIRCLE,
-			new_markup_lists[SGF_MARKUP_CIRCLE]));
+			new_markup_lists[SGF_MARKUP_CIRCLE], side_effect));
   anything_changed |= (sgf_utils_set_list_of_point_property
 		       (tree->current_node, tree, SGF_SQUARE,
-			new_markup_lists[SGF_MARKUP_SQUARE]));
+			new_markup_lists[SGF_MARKUP_SQUARE], side_effect));
   anything_changed |= (sgf_utils_set_list_of_point_property
 		       (tree->current_node, tree, SGF_TRIANGLE,
-			new_markup_lists[SGF_MARKUP_TRIANGLE]));
+			new_markup_lists[SGF_MARKUP_TRIANGLE], side_effect));
   anything_changed |= (sgf_utils_set_list_of_point_property
 		       (tree->current_node, tree, SGF_SELECTED,
-			new_markup_lists[SGF_MARKUP_SELECTED]));
+			new_markup_lists[SGF_MARKUP_SELECTED], side_effect));
 
   sgf_utils_end_action (tree);
 
@@ -559,7 +564,8 @@ sgf_utils_apply_markup_changes (SgfGameTree *tree,
 
 
 int
-sgf_utils_set_none_property (SgfNode *node, SgfGameTree *tree, SgfType type)
+sgf_utils_set_none_property (SgfNode *node, SgfGameTree *tree, SgfType type,
+			     int side_effect)
 {
   SgfProperty **link;
   SgfUndoHistoryEntry *entry;
@@ -573,7 +579,8 @@ sgf_utils_set_none_property (SgfNode *node, SgfGameTree *tree, SgfType type)
 
   sgf_utils_begin_action (tree);
 
-  entry = sgf_new_property_undo_history_entry_new (tree, node, link, type);
+  entry = sgf_new_property_undo_history_entry_new (tree, node, link, type,
+						   side_effect);
   sgf_utils_apply_undo_history_entry (tree, entry);
 
   sgf_utils_end_action (tree);
@@ -584,27 +591,43 @@ sgf_utils_set_none_property (SgfNode *node, SgfGameTree *tree, SgfType type)
 
 int
 sgf_utils_set_number_property (SgfNode *node, SgfGameTree *tree, SgfType type,
-			       int number)
+			       int number, int side_effect)
 {
   SgfProperty **link;
   SgfUndoHistoryEntry *entry;
 
   assert (node);
   assert (tree);
-  assert (property_info[type].value_type == SGF_NUMBER
-	  || property_info[type].value_type == SGF_DOUBLE
-	  || property_info[type].value_type == SGF_COLOR);
 
-  if (sgf_node_find_property (node, type, &link)) {
-    if ((*link)->value.number == number)
-      return 0;
+  if (type != SGF_TO_PLAY) {
+    assert (property_info[type].value_type == SGF_NUMBER
+	    || property_info[type].value_type == SGF_DOUBLE
+	    || property_info[type].value_type == SGF_COLOR);
 
-    entry = sgf_change_property_undo_history_entry_new (node, *link);
-    ((SgfChangePropertyOperationEntry *) entry)->value.number = number;
+    if (sgf_node_find_property (node, type, &link)) {
+      if ((*link)->value.number == number)
+	return 0;
+
+      entry = sgf_change_property_undo_history_entry_new (node, *link,
+							  side_effect);
+      ((SgfChangePropertyOperationEntry *) entry)->value.number = number;
+    }
+    else {
+      entry = sgf_new_property_undo_history_entry_new (tree, node, link, type,
+						       side_effect);
+      ((SgfPropertyOperationEntry *) entry)->property->value.number = number;
+    }
   }
   else {
-    entry = sgf_new_property_undo_history_entry_new (tree, node, link, type);
-    ((SgfPropertyOperationEntry *) entry)->property->value.number = number;
+    if (node->to_play_color == number)
+      return 0;
+
+    assert (number == EMPTY
+	    || (!IS_STONE (node->move_color) && IS_STONE (number)));
+
+    entry = (sgf_change_node_inlined_color_undo_history_entry_new
+	     (node, SGF_OPERATION_CHANGE_NODE_TO_PLAY_COLOR, number,
+	      side_effect));
   }
 
   sgf_utils_begin_action (tree);
@@ -617,7 +640,7 @@ sgf_utils_set_number_property (SgfNode *node, SgfGameTree *tree, SgfType type,
 
 int
 sgf_utils_set_real_property (SgfNode *node, SgfGameTree *tree, SgfType type,
-			     double value)
+			     double value, int side_effect)
 {
   SgfProperty **link;
   SgfUndoHistoryEntry *entry;
@@ -633,20 +656,23 @@ sgf_utils_set_real_property (SgfNode *node, SgfGameTree *tree, SgfType type,
       return 0;
 
     entry = sgf_change_real_property_undo_history_entry_new (node, *link,
-							     value);
+							     value,
+							     side_effect);
 
 #else /* not SGF_REAL_VALUES_ALLOCATED_SEPARATELY */
 
     if ((*link)->value.real == value)
       return 0;
 
-    entry = sgf_change_property_undo_history_entry_new (node, *link);
+    entry = sgf_change_property_undo_history_entry_new (node, *link,
+							side_effect);
     ((SgfChangePropertyOperationEntry *) entry)->value.real = value;
 
 #endif /* not SGF_REAL_VALUES_ALLOCATED_SEPARATELY */
   }
   else {
-    entry = sgf_new_property_undo_history_entry_new (tree, node, link, type);
+    entry = sgf_new_property_undo_history_entry_new (tree, node, link, type,
+						     side_effect);
 
 #if SGF_REAL_VALUES_ALLOCATED_SEPARATELY
     ((SgfPropertyOperationEntry *) entry)->property->value.real
@@ -666,20 +692,22 @@ sgf_utils_set_real_property (SgfNode *node, SgfGameTree *tree, SgfType type,
 
 int
 sgf_utils_set_text_property (SgfNode *node, SgfGameTree *tree, SgfType type,
-			     char *text)
+			     char *text, int side_effect)
 {
   assert (property_info[type].value_type == SGF_SIMPLE_TEXT
 	  || property_info[type].value_type == SGF_FAKE_SIMPLE_TEXT
 	  || property_info[type].value_type == SGF_TEXT);
 
-  return do_set_pointer_property (node, tree, type, strings_are_equal, text);
+  return do_set_pointer_property (node, tree, type, strings_are_equal, text,
+				  side_effect);
 }
 
 
 int
 sgf_utils_set_list_of_point_property (SgfNode *node, SgfGameTree *tree,
 				      SgfType type,
-				      BoardPositionList *position_list)
+				      BoardPositionList *position_list,
+				      int side_effect)
 {
   assert (property_info[type].value_type == SGF_LIST_OF_POINT
 	  || property_info[type].value_type == SGF_ELIST_OF_POINT);
@@ -687,26 +715,86 @@ sgf_utils_set_list_of_point_property (SgfNode *node, SgfGameTree *tree,
   return do_set_pointer_property (node, tree, type,
 				  ((ValuesComparator)
 				   board_position_lists_are_equal),
-				  position_list);
+				  position_list, side_effect);
 }
 
 
 int
 sgf_utils_set_list_of_label_property (SgfNode *node, SgfGameTree *tree,
 				      SgfType type,
-				      SgfLabelList *label_list)
+				      SgfLabelList *label_list,
+				      int side_effect)
 {
   assert (property_info[type].value_type == SGF_LIST_OF_LABEL);
 
   return do_set_pointer_property (node, tree, type,
 				  (ValuesComparator) sgf_label_lists_are_equal,
-				  label_list);
+				  label_list, side_effect);
+}
+
+
+/* Add a text property to a given node.  If such a property already
+ * exists, don't overwrite its value, but instead append new value to
+ * the old one, putting `separator' in between.  If this property
+ * doesn't yet exist, the `separator' is not used.  If `separator' is
+ * NULL, then "\n\n" is used.
+ *
+ * Return non-zero if the value is appended (zero if added).
+ */
+int
+sgf_utils_append_text_property (SgfNode *node, SgfGameTree *tree, SgfType type,
+				char *text, const char *separator,
+				int side_effect)
+{
+  const char *existing_text;
+
+  assert (node);
+  assert (property_info[type].value_type == SGF_SIMPLE_TEXT
+	  || property_info[type].value_type == SGF_FAKE_SIMPLE_TEXT
+	  || property_info[type].value_type == SGF_TEXT);
+
+  existing_text = sgf_node_get_text_property_value (node, type);
+
+  if (existing_text) {
+    char *text_to_free = text;
+
+    text = utils_cat_strings (utils_duplicate_string (existing_text),
+			      (separator ? separator : "\n\n"), text, NULL);
+    utils_free (text_to_free);
+  }
+
+  return do_set_pointer_property (node, tree, type, strings_are_equal, text,
+				  side_effect);
+}
+
+
+int
+sgf_utils_set_score_result (SgfNode *node, SgfGameTree *tree, double score,
+			    int side_effect)
+{
+  char *result;
+
+  if (score < -0.000005 || 0.000005 < score) {
+    char color_who_won = (score > 0.0 ? 'B' : 'W');
+
+    if (tree->game == GAME_GO
+	&& fabs (score - floor (score + 0.000005)) >= 0.000005)
+      result = utils_cprintf ("%c+%.f", color_who_won, fabs (score));
+    else
+      result = utils_cprintf ("%c+%d", color_who_won, (int) fabs (score));
+  }
+  else
+    result = utils_duplicate_string ("Draw");
+
+  return do_set_pointer_property (node, tree, SGF_RESULT,
+				  strings_are_equal, result, side_effect);
 }
 
 
 int
 sgf_utils_set_time_left (SgfNode *node, SgfGameTree *tree,
-			 int color, double time_left, int moves_left)
+			 int color, double time_left, int moves_left,
+			 int side_effect)
 {
   assert (node);
   assert (tree);
@@ -718,14 +806,14 @@ sgf_utils_set_time_left (SgfNode *node, SgfGameTree *tree,
 			       (color == BLACK
 				? SGF_TIME_LEFT_FOR_BLACK
 				: SGF_TIME_LEFT_FOR_WHITE),
-			       time_left);
+			       time_left, side_effect);
 
   if (moves_left) {
     sgf_utils_set_number_property (node, tree,
 				   (color == BLACK
 				    ? SGF_MOVES_LEFT_FOR_BLACK
 				    : SGF_MOVES_LEFT_FOR_WHITE),
-				   moves_left);
+				   moves_left, side_effect);
   }
 
   sgf_utils_end_action (tree);
@@ -739,6 +827,116 @@ sgf_utils_set_time_left (SgfNode *node, SgfGameTree *tree,
 }
 
 
+void
+sgf_utils_apply_custom_undo_entry
+  (SgfGameTree *tree, const SgfCustomUndoHistoryEntryData *entry_data,
+   void *user_data, SgfNode *node_to_switch_to)
+{
+  SgfUndoHistoryEntry *entry;
+
+  assert (tree);
+  assert (entry_data);
+
+  entry = sgf_custom_undo_history_entry_new (entry_data, user_data,
+					     node_to_switch_to);
+
+  sgf_utils_begin_action (tree);
+  sgf_utils_apply_undo_history_entry (tree, entry);
+  sgf_utils_end_action (tree);
+}
+
+
+int
+sgf_utils_get_node_move_number (const SgfNode *node, const SgfGameTree *tree)
+{
+  int num_moves;
+
+  assert (node);
+  assert (IS_STONE (node->move_color));
+  assert (tree);
+
+  for (num_moves = 0; node && node != tree->current_node;
+       node = node->parent) {
+    int move_number;
+
+    if (sgf_node_get_number_property_value (node, SGF_MOVE_NUMBER,
+					    &move_number))
+      return move_number + num_moves;
+
+    if (IS_STONE (node->move_color))
+      num_moves++;
+  }
+
+  if (!node)
+    return num_moves;
+  else
+    return tree->board->move_number + num_moves;
+}
+
+
+int
+sgf_utils_get_sequential_move_number (const SgfGameTree *tree)
+{
+  int move_number;
+
+  assert (tree);
+  assert (tree->current_node);
+  assert (tree->board);
+
+  if (sgf_node_find_property (tree->current_node, SGF_MOVE_NUMBER, NULL)) {
+    if (tree->current_node->parent)
+      move_number = board_get_move_number (tree->board, 1);
+    else
+      move_number = 0;
+
+    if (IS_STONE (tree->current_node->move_color))
+      move_number++;
+  }
+  else
+    move_number = tree->board->move_number;
+
+  return move_number;
+}
+
+
+int
+sgf_utils_determine_player_to_move_by_rules (const SgfGameTree *tree)
+{
+  int color_to_play_copy;
+  int sgf_color_to_play_copy;
+  int color_to_play_by_rules;
+
+  assert (tree);
+  assert (tree->current_node);
+
+  color_to_play_copy	 = tree->board_state->color_to_play;
+  sgf_color_to_play_copy = tree->board_state->sgf_color_to_play;
+
+  find_board_state_data (tree, tree->current_node, 1, 0);
+  determine_final_color_to_play (tree);
+
+  color_to_play_by_rules = tree->board_state->color_to_play;
+
+  tree->board_state->color_to_play     = color_to_play_copy;
+  tree->board_state->sgf_color_to_play = sgf_color_to_play_copy;
+
+  return color_to_play_by_rules;
+}
+
+
+void
+sgf_utils_find_board_state_data (const SgfGameTree *tree,
+				 int need_color_to_move,
+				 int need_move_coordinates)
+{
+  assert (tree);
+  assert (tree->current_node);
+
+  find_board_state_data (tree, tree->current_node,
+			 need_color_to_move, need_move_coordinates);
+}
+
+
 void
 sgf_utils_find_time_control_data (const SgfGameTree *tree)
 {
@@ -788,8 +986,28 @@ sgf_utils_delete_current_node_children (SgfGameTree *tree)
 }
 
 
+void
+sgf_utils_swap_current_node_with (SgfGameTree *tree, SgfNode *swap_with)
+{
+  SgfUndoHistoryEntry *entry;
+
+  assert (tree);
+  assert (tree->current_node);
+  assert (swap_with);
+  assert (tree->current_node != swap_with);
+  assert (tree->current_node->parent == swap_with->parent);
+
+  entry = sgf_swap_nodes_undo_history_entry_new (tree->current_node,
+						 swap_with);
+  sgf_utils_begin_action (tree);
+  sgf_utils_apply_undo_history_entry (tree, entry);
+  sgf_utils_end_action (tree);
+}
+
+
 int
-sgf_utils_delete_property (SgfNode *node, SgfGameTree *tree, SgfType type)
+sgf_utils_delete_property (SgfNode *node, SgfGameTree *tree, SgfType type,
+			   int side_effect)
 {
   SgfProperty **link;
 
@@ -798,7 +1016,7 @@ sgf_utils_delete_property (SgfNode *node, SgfGameTree *tree, SgfType type)
 
   if (sgf_node_find_property (node, type, &link)) {
     SgfUndoHistoryEntry *entry
-      = sgf_delete_property_undo_history_entry_new (node, *link);
+      = sgf_delete_property_undo_history_entry_new (node, *link, side_effect);
 
     sgf_utils_begin_action (tree);
     sgf_utils_apply_undo_history_entry (tree, entry);
@@ -1073,6 +1291,390 @@ sgf_utils_normalize_text (const char *text, int is_simple_text)
 }
 
 
+/* Create and return an in-memory SGF of a subtree of `tree', starting
+ * at `subtree_root'.  The resulting SGF tree always has extra dummy
+ * root, which is used to setup board.  I.e. `subtree_root' will
+ * correspond to the only child of SGF game tree root.
+ */
+char *
+sgf_utils_create_subtree_sgf (SgfGameTree *tree, SgfNode *subtree_root,
+			      int *sgf_length)
+{
+  SgfNode *current_node;
+  SgfCollection *pseudo_collection;
+  SgfGameTree *pseudo_tree;
+  SgfNode *pseudo_root;
+  SgfNode *pseudo_subtree_root;
+  Board *board;
+  SgfBoardState board_state;
+  char *result;
+
+  assert (tree);
+  assert (tree->board);
+  assert (tree->board_state);
+  assert (subtree_root);
+  assert (sgf_length);
+
+  pseudo_collection = sgf_collection_new ();
+  pseudo_tree	    = sgf_game_tree_new_with_root (tree->game,
+						   tree->board_width,
+						   tree->board_height,
+						   0);
+  pseudo_root	    = pseudo_tree->root;
+
+  sgf_collection_add_game_tree (pseudo_collection, pseudo_tree);
+
+  current_node = tree->current_node;
+  if (subtree_root != current_node)
+    sgf_utils_do_switch_to_given_node (tree, subtree_root);
+
+  board = board_new (tree->game, tree->board_width, tree->board_height);
+  sgf_utils_enter_tree (pseudo_tree, board, &board_state);
+
+  /* Copy board grid state to the pseudo tree. */
+  sgf_utils_apply_setup_changes (pseudo_tree, tree->board->grid, 1);
+
+  /* FIXME: Should the board be deleted by the tree itself? */
+  board_delete (board);
+
+  if (current_node != subtree_root)
+    sgf_utils_do_switch_to_given_node (tree, current_node);
+
+  pseudo_subtree_root	     = sgf_node_duplicate (subtree_root, pseudo_tree,
+						   pseudo_root);
+  pseudo_subtree_root->child = subtree_root->child;
+  pseudo_root->child	     = pseudo_subtree_root;
+
+  result = sgf_write_in_memory (pseudo_collection, 1, sgf_length);
+
+  /* Don't accidentally delete real tree's nodes! */
+  pseudo_subtree_root->child = NULL;
+
+  sgf_collection_delete (pseudo_collection);
+
+  return result;
+}
+
+
+SgfPasteResult
+sgf_utils_paste_sgf (SgfGameTree *tree, SgfNode *parent_node,
+		     char *sgf, int sgf_length)
+{
+  SgfCollection *parsed_collection;
+  SgfGameTree *parsed_tree;
+  SgfErrorList *errors;
+  SgfNode *node_to_paste;
+  int parsing_result;
+
+  assert (tree);
+  assert (tree->current_node);
+  assert (parent_node);
+  assert (sgf);
+  assert (sgf_length > 0);
+
+  parsing_result = sgf_parse_buffer (sgf, sgf_length, &parsed_collection,
+				     &errors, &sgf_parser_defaults,
+				     NULL, NULL);
+
+  /* FIXME: Errors are ignored. */
+  if (errors)
+    string_list_delete (errors);
+
+  switch (parsing_result) {
+  case SGF_PARSING_CANCELLED:
+  case SGF_ERROR_READING_FILE:
+    assert (0);
+
+  case SGF_INVALID_FILE:
+    return SGF_INVALID_SGF;
+  }
+
+  if (parsed_collection->num_trees != 1) {
+    sgf_collection_delete (parsed_collection);
+    return SGF_NOT_CLIPBOARD_SGF;
+  }
+
+  parsed_tree = parsed_collection->first_tree;
+
+  /* A valid SGF to paste must have exactly one root child. */
+  if (!parsed_tree->root
+      || !parsed_tree->root->child
+      || parsed_tree->root->child->next) {
+    sgf_collection_delete (parsed_collection);
+    return SGF_NOT_CLIPBOARD_SGF;
+  }
+
+  if (parsed_tree->game		   != tree->game
+      || parsed_tree->board_width  != tree->board_width
+      || parsed_tree->board_height != tree->board_height) {
+    sgf_collection_delete (parsed_collection);
+    return SGF_COULDNT_PASTE;
+  }
+
+  /* This is quite ugly, but we have to duplicate everything for the
+   * tree we paste into, because node and property allocation is
+   * tree-specific with memory pools (the default.)
+   */
+  node_to_paste = sgf_node_duplicate_recursively (parsed_tree->root->child,
+						  tree, parent_node);
+  sgf_collection_delete (parsed_collection);
+
+  if (parent_node != tree->current_node)
+    sgf_utils_do_switch_to_given_node (tree, parent_node);
+
+  sgf_utils_begin_action (tree);
+  sgf_utils_apply_undo_history_entry
+    (tree, sgf_new_node_undo_history_entry_new (node_to_paste));
+  sgf_utils_end_action (tree);
+
+  return SGF_PASTED;
+}
+
+
+char *
+sgf_utils_export_position_as_ascii (const SgfGameTree *tree)
+{
+  StringBuffer result;
+  BoardPoint hoshi_points[9];
+  int num_hoshi_points = 0;
+  int x;
+  int y;
+
+  assert (tree);
+  assert (tree->board);
+  assert (tree->board_state);
+
+  if (tree->game == GAME_GO) {
+    num_hoshi_points = go_get_hoshi_points (tree->board_width,
+					    tree->board_height,
+					    hoshi_points);
+  }
+
+  string_buffer_init (&result, 0x400, 0x200);
+
+  add_horizontal_coordinates (tree, &result);
+  add_horizontal_line (tree, &result, 0);
+
+  for (y = 0; y < tree->board_height; y++) {
+    int vertical_coordinate
+      = (game_info[tree->game].reversed_vertical_coordinates
+	 ? tree->board_height - y : y + 1);
+
+    string_buffer_printf (&result, "%*d |",
+			  tree->board_height < 10 ? 1 : 2,
+			  vertical_coordinate);
+
+    if (tree->board_state->last_move_x != 0
+	|| tree->board_state->last_move_y != y)
+      string_buffer_add_character (&result, ' ');
+    else
+      string_buffer_add_character (&result, '(');
+
+    for (x = 0; x < tree->board_width; x++) {
+      char character = '.';
+
+      switch (tree->board->grid[POSITION (x, y)]) {
+      case EMPTY:
+	{
+	  int k;
+
+	  for (k = 0; k < num_hoshi_points; k++) {
+	    if (x == hoshi_points[k].x && y == hoshi_points[k].y) {
+	      character = '+';
+	      break;
+	    }
+	  }
+	}
+
+	break;
+
+      case BLACK:
+	character = 'X';
+	break;
+
+      case WHITE:
+	character = 'O';
+	break;
+
+      case SPECIAL_ON_GRID_VALUE:
+	character = '=';
+	break;
+
+      default:
+	assert (0);
+      }
+
+      string_buffer_add_character (&result, character);
+
+      if ((tree->board_state->last_move_x != x
+	   && tree->board_state->last_move_x != x + 1)
+	  || tree->board_state->last_move_y != y)
+	string_buffer_add_character (&result, ' ');
+      else {
+	if (tree->board_state->last_move_x == x)
+	  string_buffer_add_character (&result, ')');
+	else
+	  string_buffer_add_character (&result, '(');
+      }
+    }
+
+    string_buffer_printf (&result, "| %-*d\n",
+			  tree->board_height < 10 ? 1 : 2,
+			  vertical_coordinate);
+  }
+
+  add_horizontal_line (tree, &result, 0);
+  add_horizontal_coordinates (tree, &result);
+
+  return string_buffer_steal_string (&result);
+}
+
+
+char *
+sgf_utils_export_position_as_senseis_library_diagram (const SgfGameTree *tree)
+{
+  StringBuffer result;
+  const SgfBoardState *board_state;
+  const BoardPositionList* circle_mark;
+  const BoardPositionList* square_mark;
+  const SgfLabelList *labels;
+  int x;
+  int y;
+
+  assert (tree);
+  assert (tree->game == GAME_GO);
+  assert (tree->board);
+  assert (tree->board_state);
+  assert (tree->current_node);
+
+  board_state = tree->board_state;
+  circle_mark = sgf_node_get_list_of_point_property_value (tree->current_node,
+							   SGF_CIRCLE);
+  square_mark = sgf_node_get_list_of_point_property_value (tree->current_node,
+							   SGF_SQUARE);
+  labels      = sgf_node_get_list_of_label_property_value (tree->current_node,
+							   SGF_LABEL);
+
+  string_buffer_init (&result, 0x400, 0x200);
+
+  string_buffer_add_characters (&result, '$', 2);
+
+  if (board_state->last_move_node) {
+    string_buffer_add_character
+      (&result, board_state->last_move_node->move_color == BLACK ? 'B' : 'W');
+  }
+
+  string_buffer_cat_string (&result, " \n");
+
+  add_horizontal_line (tree, &result, 1);
+
+  for (y = 0; y < tree->board_height; y++) {
+    string_buffer_cat_string (&result, "$$ |");
+
+    for (x = 0; x < tree->board_width; x++) {
+      static const char *characters = ".CSXB#OW@1";
+
+      int pos = POSITION (x, y);
+      const char *character;
+
+      string_buffer_add_character (&result, ' ');
+
+      if (tree->board->grid[pos] == EMPTY) {
+	if (labels) {
+	  BoardPoint point = { x, y };
+	  const char *label = sgf_label_list_get_label (labels, point);
+
+	  if (label
+	      && (('a' <= label[0] && label[0] <= 'z')
+		  || ('A' <= label[0] && label[0] <= 'Z'))
+	      && !label[1]) {
+	    char label_character = label[0];
+
+	    if ('A' <= label_character && label_character <= 'Z')
+	      label_character -= 'A' - 'a';
+
+	    string_buffer_add_character (&result, label_character);
+	    continue;
+	  }
+	}
+
+	character = characters;
+      }
+      else {
+	switch (tree->board->grid[pos]) {
+	case BLACK:
+	  character = characters + 3;
+	  break;
+
+	case WHITE:
+	  character = characters + 6;
+	  break;
+
+	default:
+	  assert (0);
+	}
+      }
+
+      if (circle_mark
+	  && board_position_list_find_position (circle_mark, pos) != -1)
+	character += 1;
+      else if (square_mark
+	       && board_position_list_find_position (square_mark, pos) != -1)
+	character += 2;
+      else if (IS_STONE (tree->board->grid[pos])
+	       && tree->board_state->last_move_x == x
+	       && tree->board_state->last_move_y == y)
+	character = characters + 9;
+
+      string_buffer_add_character (&result, *character);
+    }
+
+    string_buffer_cat_string (&result, " |\n");
+  }
+
+  add_horizontal_line (tree, &result, 1);
+
+  return string_buffer_steal_string (&result);
+}
+
+
+static void
+add_horizontal_coordinates (const SgfGameTree *tree, StringBuffer *buffer)
+{
+  int x;
+
+  string_buffer_add_characters (buffer, ' ', tree->board_height < 10 ? 3 : 4);
+
+  for (x = 0; x < tree->board_width; x++) {
+    char coordinate_character
+      = game_info[tree->game].horizontal_coordinates[x];
+
+    string_buffer_add_character (buffer, ' ');
+    string_buffer_add_character (buffer, coordinate_character);
+  }
+
+  string_buffer_add_character (buffer, '\n');
+}
+
+
+static void
+add_horizontal_line (const SgfGameTree *tree, StringBuffer *buffer,
+		     int sensei_style)
+{
+  if (sensei_style)
+    string_buffer_cat_string (buffer, "$$ ");
+  else {
+    string_buffer_add_characters (buffer, ' ',
+				  tree->board_height < 10 ? 2 : 3);
+  }
+
+  string_buffer_add_character (buffer, '+');
+  string_buffer_add_characters (buffer, '-', 2 * tree->board_width + 1);
+  string_buffer_add_character (buffer, '+');
+  string_buffer_add_character (buffer, '\n');
+}
+
+
 
 
 static void
@@ -1171,8 +1773,6 @@ sgf_utils_descend_nodes (SgfGameTree *tree, int num_nodes)
       board_state->last_move_node    = node;
     }
     else {
-      int color;
-
       if (node->move_color == SETUP_NODE) {
 	const BoardPositionList *position_lists[NUM_ON_GRID_VALUES];
 
@@ -1193,13 +1793,12 @@ sgf_utils_descend_nodes (SgfGameTree *tree, int num_nodes)
 	board_apply_changes (tree->board, position_lists);
 
 	board_state->last_move_x = NULL_X;
-
-	color = sgf_node_get_color_property_value (node, SGF_TO_PLAY);
-	if (color != EMPTY)
-	  board_state->sgf_color_to_play = color;
       }
       else
 	board_add_dummy_move_entry (tree->board);
+
+      if (node->to_play_color != EMPTY)
+	board_state->sgf_color_to_play = node->to_play_color;
     }
 
     if (node->move_color != SETUP_NODE) {
@@ -1237,50 +1836,9 @@ sgf_utils_ascend_nodes (SgfGameTree *tree, int num_nodes)
   }
 
   if (node->parent) {
-    SgfNode *look_up_node = node;
-    int need_move_coordinates = 1;
-
     board_undo (tree->board, num_nodes);
 
-    board_state->sgf_color_to_play = EMPTY;
-    board_state->last_move_node    = NULL;
-
-    while (1) {
-      if (IS_STONE (look_up_node->move_color)) {
-	if (board_state->sgf_color_to_play == EMPTY) {
-	  board_state->sgf_color_to_play
-	    = OTHER_COLOR (look_up_node->move_color);
-	}
-
-	board_state->last_move_node = look_up_node;
-	if (need_move_coordinates) {
-	  board_state->last_move_x = look_up_node->move_point.x;
-	  board_state->last_move_y = look_up_node->move_point.y;
-	}
-
-	break;
-      }
-      else if (look_up_node->move_color == SETUP_NODE) {
-	if (board_state->sgf_color_to_play == EMPTY) {
-	  /* `PL' is a setup property, so we check only here. */
-	  board_state->sgf_color_to_play
-	    = sgf_node_get_color_property_value (look_up_node, SGF_TO_PLAY);
-	}
-
-	board_state->last_move_x = NULL_X;
-	board_state->last_move_y = NULL_Y;
-	need_move_coordinates	 = 0;
-      }
-
-      look_up_node = look_up_node->parent;
-      if (!look_up_node) {
-	board_state->last_move_x = NULL_X;
-	board_state->last_move_y = NULL_Y;
-
-	break;
-      }
-    }
-
+    find_board_state_data (tree, node, 1, 1);
     find_time_control_data (tree, NULL, node);
 
     tree->current_node	      = node;
@@ -1297,6 +1855,60 @@ sgf_utils_ascend_nodes (SgfGameTree *tree, int num_nodes)
 }
 
 
+static void
+find_board_state_data (const SgfGameTree *tree, SgfNode *node,
+		       int need_color_to_move, int need_move_coordinates)
+{
+  SgfBoardState *const board_state = tree->board_state;
+
+  if (!need_color_to_move && !need_color_to_move)
+    return;
+
+  board_state->last_move_node = NULL;
+  if (need_color_to_move)
+    board_state->sgf_color_to_play = EMPTY;
+
+  while (1) {
+    if (IS_STONE (node->move_color)) {
+      if (board_state->sgf_color_to_play == EMPTY)
+	board_state->sgf_color_to_play = OTHER_COLOR (node->move_color);
+
+      board_state->last_move_node = node;
+      if (need_move_coordinates) {
+	board_state->last_move_x = node->move_point.x;
+	board_state->last_move_y = node->move_point.y;
+      }
+
+      break;
+    }
+    else {
+      if (board_state->sgf_color_to_play == EMPTY) {
+	/* `PL' is a setup property, so we check only here. */
+	board_state->sgf_color_to_play = node->to_play_color;
+      }
+
+      if (node->move_color == SETUP_NODE) {
+	if (need_move_coordinates) {
+	  board_state->last_move_x = NULL_X;
+	  board_state->last_move_y = NULL_Y;
+	  need_move_coordinates	   = 0;
+	}
+      }
+    }
+
+    node = node->parent;
+    if (!node) {
+      if (need_move_coordinates) {
+	board_state->last_move_x = NULL_X;
+	board_state->last_move_y = NULL_Y;
+      }
+
+      break;
+    }
+  }
+}
+
+
 static void
 find_time_control_data (const SgfGameTree *tree,
 			SgfNode *upper_limit, SgfNode *lower_limit)
@@ -1356,7 +1968,7 @@ find_time_control_data (const SgfGameTree *tree,
 
 
 static void
-determine_final_color_to_play (SgfGameTree *tree)
+determine_final_color_to_play (const SgfGameTree *tree)
 {
   const SgfNode *game_info_node = tree->board_state->game_info_node;
 
@@ -1383,7 +1995,8 @@ determine_final_color_to_play (SgfGameTree *tree)
 
 static int
 do_set_pointer_property (SgfNode *node, SgfGameTree *tree, SgfType type,
-			 ValuesComparator values_are_equal, void *new_value)
+			 ValuesComparator values_are_equal, void *new_value,
+			 int side_effect)
 {
   SgfProperty **link;
   SgfUndoHistoryEntry *entry;
@@ -1401,19 +2014,22 @@ do_set_pointer_property (SgfNode *node, SgfGameTree *tree, SgfType type,
 	return 0;
       }
 
-      entry = sgf_change_property_undo_history_entry_new (node, *link);
+      entry = sgf_change_property_undo_history_entry_new (node, *link,
+							  side_effect);
 
       ((SgfChangePropertyOperationEntry *) entry)->value.memory_block
 	= new_value;
     }
     else
-      entry = sgf_delete_property_undo_history_entry_new (node, *link);
+      entry = sgf_delete_property_undo_history_entry_new (node, *link,
+							  side_effect);
   }
   else {
     if (!new_value)
       return 0;
 
-    entry = sgf_new_property_undo_history_entry_new (tree, node, link, type);
+    entry = sgf_new_property_undo_history_entry_new (tree, node, link, type,
+						     side_effect);
     ((SgfPropertyOperationEntry *) entry)->property->value.memory_block
       = new_value;
   }
