@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
  * This file is part of Quarry.                                    *
  *                                                                 *
- * Copyright (C) 2003, 2004, 2005 Paul Pogonyshev                  *
+ * Copyright (C) 2003, 2004, 2005, 2006 Paul Pogonyshev.           *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -25,14 +25,24 @@
 
 
 #include "sgf-properties.h"
-#include "utils.h"
 #include "board.h"
+#include "utils.h"
 #include "quarry.h"
 
 
 
 /* `sgf-tree.c' global declarations and functions. */
 
+/* We don't use Go-specific MIME type here and I think it is right,
+ * since we mean generic SGF here.
+ */
+#define SGF_MIME_TYPE		"text/x-sgf"
+
+
+/* Note that in this means that the node has one of the `add sth.'
+ * properties.  Other setup (in SGF sense) nodes, like `PL' do not
+ * count!
+ */
 #define SETUP_NODE		SPECIAL_ON_GRID_VALUE
 
 
@@ -91,41 +101,47 @@ typedef enum {
 } SgfGameTreeNotificationCode;
 
 
-typedef struct _SgfVector		SgfVector;
-typedef struct _SgfVectorList		SgfVectorList;
+typedef struct _SgfVector			SgfVector;
+typedef struct _SgfVectorList			SgfVectorList;
 
-typedef struct _SgfLabel		SgfLabel;
-typedef struct _SgfLabelList		SgfLabelList;
+typedef struct _SgfLabel			SgfLabel;
+typedef struct _SgfLabelList			SgfLabelList;
 
-typedef struct _SgfFigureDescription	SgfFigureDescription;
+typedef struct _SgfFigureDescription		SgfFigureDescription;
 
-typedef union  _SgfValue		SgfValue;
-typedef struct _SgfProperty		SgfProperty;
+typedef union  _SgfValue			SgfValue;
+typedef struct _SgfProperty			SgfProperty;
 
-typedef struct _SgfNode			SgfNode;
-typedef struct _SgfNodeGeneric		SgfNodeGeneric;
-typedef struct _SgfNodeGeneric		SgfNodeGo;
-typedef struct _SgfNodeGeneric		SgfNodeReversi;
-typedef struct _SgfNodeAmazons		SgfNodeAmazons;
+typedef struct _SgfNode				SgfNode;
+typedef struct _SgfNodeGeneric			SgfNodeGeneric;
+typedef struct _SgfNodeGeneric			SgfNodeGo;
+typedef struct _SgfNodeGeneric			SgfNodeReversi;
+typedef struct _SgfNodeAmazons			SgfNodeAmazons;
 
-typedef struct _SgfBoardState		SgfBoardState;
+typedef struct _SgfBoardState			SgfBoardState;
 
-typedef struct _SgfUndoHistoryEntry	SgfUndoHistoryEntry;
-typedef struct _SgfUndoHistory		SgfUndoHistory;
+typedef struct _SgfUndoHistoryEntry		SgfUndoHistoryEntry;
+typedef struct _SgfUndoHistory			SgfUndoHistory;
 
 typedef void (* SgfUndoHistoryNotificationCallback)
   (SgfUndoHistory *undo_history, void *user_data);
 
-typedef struct _SgfGameTreeMapData	SgfGameTreeMapData;
-typedef struct _SgfGameTreeMapLine	SgfGameTreeMapLine;
-typedef struct _SgfGameTree		SgfGameTree;
+typedef struct _SgfGameTree			SgfGameTree;
+
+typedef void (* SgfCustomOperationEntryFunction) (void *user_data,
+						  SgfGameTree *tree);
+
+typedef struct _SgfCustomUndoHistoryEntryData	SgfCustomUndoHistoryEntryData;
+
+typedef struct _SgfGameTreeMapData		SgfGameTreeMapData;
+typedef struct _SgfGameTreeMapLine		SgfGameTreeMapLine;
 
 typedef void (* SgfGameTreeNotificationCallback)
   (SgfGameTree *tree, SgfGameTreeNotificationCode notification_code,
    void *user_data);
 
 
-typedef struct _SgfCollection		SgfCollection;
+typedef struct _SgfCollection			SgfCollection;
 
 typedef void (* SgfCollectionNotificationCallback) (SgfCollection *collection,
 						    void *user_data);
@@ -226,6 +242,8 @@ struct _SgfNode {
   unsigned int		  is_collapsed : 1;
   unsigned int		  has_intermediate_map_data : 1;
 
+  unsigned int		  to_play_color : 2;
+
   unsigned int		  move_color : 2;
   BoardPoint		  move_point;
 
@@ -250,6 +268,8 @@ struct _SgfNodeGeneric {
   unsigned int		  is_collapsed : 1;
   unsigned int		  has_intermediate_map_data : 1;
 
+  unsigned int		  to_play_color : 2;
+
   unsigned int		  move_color : 2;
   BoardPoint		  move_point;
 
@@ -267,6 +287,8 @@ struct _SgfNodeAmazons {
   unsigned int		  is_collapsed : 1;
   unsigned int		  has_intermediate_map_data : 1;
 
+  unsigned int		  to_play_color : 2;
+
   unsigned int		  move_color : 2;
   BoardPoint		  move_point;
 
@@ -323,6 +345,13 @@ struct _SgfUndoHistory {
 };
 
 
+struct _SgfCustomUndoHistoryEntryData {
+  SgfCustomOperationEntryFunction   undo;
+  SgfCustomOperationEntryFunction   redo;
+  SgfCustomOperationEntryFunction   free_data;
+};
+
+
 struct _SgfGameTreeMapData {
   SgfGameTreeMapData	 *next;
 
@@ -598,16 +627,6 @@ int		 sgf_node_add_pointer_property (SgfNode *node,
 				 (overwrite))
 
 
-int		 sgf_node_add_score_result (SgfNode *node, SgfGameTree *tree,
-					    double score, int overwrite);
-
-int		 sgf_node_append_text_property (SgfNode *node,
-						SgfGameTree *tree,
-						SgfType type,
-						char *text,
-						const char *separator);
-
-
 int		 sgf_node_delete_property (SgfNode *node, SgfGameTree *tree,
 					   SgfType type);
 
@@ -773,6 +792,8 @@ extern const SgfParserParameters	sgf_parser_defaults;
 
 int		 sgf_write_file (const char *filename,
 				 SgfCollection *collection, int force_utf8);
+char *		 sgf_write_in_memory (SgfCollection *collection,
+				      int force_utf8, int *sgf_length);
 
 
 
@@ -784,10 +805,18 @@ typedef enum {
 } SgfDirection;
 
 
+typedef enum {
+  SGF_PASTED,
+  SGF_COULDNT_PASTE,
+  SGF_NOT_CLIPBOARD_SGF,
+  SGF_INVALID_SGF
+} SgfPasteResult;
+
+
 inline void   sgf_utils_play_node_move (const SgfNode *node, Board *board);
-int	      sgf_utils_format_node_move (const SgfGameTree *tree,
+void	      sgf_utils_format_node_move (const SgfGameTree *tree,
 					  const SgfNode *node,
-					  char *buffer,
+					  StringBuffer *buffer,
 					  const char *black_string,
 					  const char *white_string,
 					  const char *pass_string);
@@ -815,48 +844,83 @@ void	      sgf_utils_switch_to_given_variation (SgfGameTree *tree,
 void	      sgf_utils_switch_to_given_node (SgfGameTree *tree,
 					      SgfNode *node);
 
-void	      sgf_utils_append_variation (SgfGameTree *tree,
+SgfNode *     sgf_utils_append_variation (SgfGameTree *tree,
 					  int color, ...);
 
 int	      sgf_utils_apply_setup_changes (SgfGameTree *tree,
-					     const char grid[BOARD_GRID_SIZE]);
+					     const char grid[BOARD_GRID_SIZE],
+					     int side_effect);
 int	      sgf_utils_apply_markup_changes
-		(SgfGameTree *tree, const char markup_grid[BOARD_GRID_SIZE]);
+		(SgfGameTree *tree, const char markup_grid[BOARD_GRID_SIZE],
+		 int side_effect);
 
 int	      sgf_utils_set_none_property (SgfNode *node, SgfGameTree *tree,
-					   SgfType type);
+					   SgfType type, int side_effect);
 int	      sgf_utils_set_number_property (SgfNode *node, SgfGameTree *tree,
-					     SgfType type, int number);
+					     SgfType type, int number,
+					     int side_effect);
 int	      sgf_utils_set_real_property (SgfNode *node, SgfGameTree *tree,
-					   SgfType type, double value);
+					   SgfType type, double value,
+					   int side_effect);
 int	      sgf_utils_set_text_property (SgfNode *node, SgfGameTree *tree,
-					   SgfType type, char *text);
+					   SgfType type, char *text,
+					   int side_effect);
 int	      sgf_utils_set_list_of_point_property
 		(SgfNode *node, SgfGameTree *tree, SgfType type,
-		 BoardPositionList *position_list);
+		 BoardPositionList *position_list, int side_effect);
 int	      sgf_utils_set_list_of_label_property (SgfNode *node,
 						    SgfGameTree *tree,
 						    SgfType type,
-						    SgfLabelList *label_list);
+						    SgfLabelList *label_list,
+						    int side_effect);
 
-#define sgf_utils_set_double_property(node, tree, type, emphasized)	\
-  sgf_utils_set_number_property ((node), (tree), (type), (emphasized))
+#define sgf_utils_set_double_property(node, tree, type, emphasized,	\
+				      side_effect)			\
+  sgf_utils_set_number_property ((node), (tree), (type), (emphasized),	\
+				 (side_effect))
 
-#define sgf_utils_set_color_property(node, tree, type, color)		\
-  sgf_utils_set_number_property ((node), (tree), (type), (color))
+#define sgf_utils_set_color_property(node, tree, type, color,		\
+				     side_effect)			\
+  sgf_utils_set_number_property ((node), (tree), (type), (color),	\
+				 (side_effect))
+
+int	      sgf_utils_append_text_property (SgfNode *node, SgfGameTree *tree,
+					      SgfType type, char *text,
+					      const char *separator,
+					      int side_effect);
 
 
+int	      sgf_utils_set_score_result (SgfNode *node, SgfGameTree *tree,
+					  double score, int side_effect);
 int	      sgf_utils_set_time_left (SgfNode *node, SgfGameTree *tree,
 				       int color,
-				       double time_left, int moves_left);
+				       double time_left, int moves_left,
+				       int side_effect);
 
+void	      sgf_utils_apply_custom_undo_entry
+		(SgfGameTree *tree,
+		 const SgfCustomUndoHistoryEntryData *entry_data,
+		 void *user_data, SgfNode *node_to_switch_to);
+
+int	      sgf_utils_get_node_move_number (const SgfNode *node,
+					      const SgfGameTree *tree);
+int	      sgf_utils_get_sequential_move_number (const SgfGameTree *tree);
+
+int	      sgf_utils_determine_player_to_move_by_rules
+		(const SgfGameTree *tree);
+void	      sgf_utils_find_board_state_data (const SgfGameTree *tree,
+					       int need_color_to_move,
+					       int need_move_coordinates);
 void	      sgf_utils_find_time_control_data (const SgfGameTree *tree);
 
 void	      sgf_utils_delete_current_node (SgfGameTree *tree);
 void	      sgf_utils_delete_current_node_children (SgfGameTree *tree);
 
+void	      sgf_utils_swap_current_node_with (SgfGameTree *tree,
+						SgfNode *swap_with);
+
 int	      sgf_utils_delete_property (SgfNode *node, SgfGameTree *tree,
-					 SgfType type);
+					 SgfType type, int side_effect);
 
 
 void	      sgf_utils_set_node_is_collapsed (SgfGameTree *tree,
@@ -878,6 +942,17 @@ void	      sgf_utils_add_free_handicap_stones
 
 char *	      sgf_utils_normalize_text (const char *text, int is_simple_text);
 
+char *	      sgf_utils_create_subtree_sgf (SgfGameTree *tree,
+					    SgfNode *subtree_root,
+					    int *sgf_length);
+SgfPasteResult
+	      sgf_utils_paste_sgf (SgfGameTree *tree, SgfNode *parent_node,
+				   char *sgf, int sgf_length);
+
+char *	      sgf_utils_export_position_as_ascii (const SgfGameTree *tree);
+char *	      sgf_utils_export_position_as_senseis_library_diagram
+		(const SgfGameTree *tree);
+
 
 
 /* `sgf-undo.c' global functions. */
@@ -891,6 +966,19 @@ void		  sgf_undo_history_set_notification_callback
 		     SgfUndoHistoryNotificationCallback notification_callback,
 		     void *user_data);
 
+void		  sgf_undo_history_hide_last_applied_entry
+		    (SgfUndoHistory *history);
+int		  sgf_undo_history_is_last_applied_entry_single
+		    (const SgfUndoHistory *history);
+int		  sgf_undo_history_check_last_applied_custom_entry_type
+		    (const SgfUndoHistory *history,
+		     const SgfCustomUndoHistoryEntryData *entry_data);
+void *		  sgf_undo_history_get_last_applied_custom_entry_data
+		    (const SgfUndoHistory *history);
+
+void		  sgf_undo_history_delete_redo_entries
+		    (SgfUndoHistory *history, SgfGameTree *tree);
+
 void		  sgf_utils_begin_action (SgfGameTree *tree);
 void		  sgf_utils_end_action (SgfGameTree *tree);
 
