@@ -3,7 +3,7 @@
  *                                                                 *
  * Copyright (C) 2003 Paul Pogonyshev.                             *
  * Copyright (C) 2004 Paul Pogonyshev and Martin Holters.          *
- * Copyright (C) 2005 Paul Pogonyshev.                             *
+ * Copyright (C) 2005, 2006 Paul Pogonyshev.                       *
  *                                                                 *
  * This program is free software; you can redistribute it and/or   *
  * modify it under the terms of the GNU General Public License as  *
@@ -369,6 +369,8 @@ void *		  string_list_steal_item (void *abstract_list,
 					  void *abstract_item);
 void *		  string_list_steal_first_item (void *abstract_list);
 
+int		  string_list_clamp_size (void *abstract_list, int max_size);
+
 int		  string_list_get_item_index (void *abstract_list,
 					      void *abstract_item);
 StringListItem *  string_list_get_item (void *abstract_list, int item_index);
@@ -465,6 +467,7 @@ void		string_buffer_delete (StringBuffer *string_buffer);
 void		string_buffer_dispose (StringBuffer *string_buffer);
 
 void		string_buffer_empty (StringBuffer *string_buffer);
+char *		string_buffer_steal_string (StringBuffer *string_buffer);
 
 
 #define string_buffer_add_character(string_buffer, character)		\
@@ -498,26 +501,40 @@ void		string_buffer_vcprintf (StringBuffer *string_buffer,
 
 /* `buffered-writer.c' declarations and global functions. */
 
-typedef struct _BufferedWriter	BufferedWriter;
+typedef struct _BufferedWriter		BufferedWriter;
+typedef struct _BufferedWriterChunkData	BufferedWriterChunkData;
 
 struct _BufferedWriter {
-  FILE	       *file;
+  FILE			   *file;
+  BufferedWriterChunkData  *first_chunk;
 
-  char	       *buffer;
-  char	       *buffer_pointer;
-  char	       *buffer_end;
+  size_t		    buffer_size;
 
-  iconv_t	iconv_handle;
+  char			   *buffer;
+  char			   *buffer_pointer;
+  char			   *buffer_end;
 
-  size_t	column;
+  iconv_t		    iconv_handle;
 
-  int		successful;
+  size_t		    column;
+
+  int			    successful;
+};
+
+struct _BufferedWriterChunkData {
+  BufferedWriterChunkData  *next_chunk;
+  size_t		    chunk_size;
 };
 
 
 int		buffered_writer_init (BufferedWriter *writer,
 				      const char *filename, size_t buffer_size);
+void		buffered_writer_init_memory (BufferedWriter *writer,
+					     size_t buffer_size);
+
 int		buffered_writer_dispose (BufferedWriter *writer);
+char *		buffered_writer_dispose_memory (BufferedWriter *writer,
+						int *data_length);
 
 #define buffered_writer_set_iconv_handle(writer, handle)	\
   ((writer)->iconv_handle = (handle))
